@@ -191,7 +191,7 @@ def calc_crr_m7p5_from_qc1ncs(q_c1n_cs, c_0=2.8):
                         ((q_c1n_cs / 140) ** 3) + ((q_c1n_cs / 137) ** 4) - c_0)
 
 
-def calc_crr_n15_from_qc1ncs(q_c1n_cs, c_0=2.8):
+def calc_crr_n15_from_qc1ncs(q_c1ncs, c_0=2.8):
     crr_m7p5 = calc_crr_m7p5_from_qc1ncs(q_c1ncs, c_0=c_0)
     msf_m = 1.09 + (q_c1ncs / 180) ** 3
     msf_max = np.clip(msf_m, None, 2.2)
@@ -327,7 +327,7 @@ def _calc_dependent_variables(sigma_v, sigma_veff, q_c, f_s, p_a, q_t, cfc):
             if abs(q_c1n[dd] - temp_q_c1n) < 0.00001 and n_val_stable:
                 break
             temp_q_c1n = q_c1n[dd]
-    return q_c1n_cs, q_c1n, fines_content, i_c, big_q
+    return q_c1n_cs, q_c1n, fines_content, i_c, big_q, ft_values
 
 
 class BoulangerIdriss2014CPT(object):
@@ -421,7 +421,7 @@ class BoulangerIdriss2014CPT(object):
             self.sigma_veff[0] = 1.0e-10
         self.rd = calc_rd(self.depth, self.m_w)
 
-        self.q_c1n_cs, self.q_c1n, self.fines_content, self.i_c, self.big_q = _calc_dependent_variables(self.sigma_v,
+        self.q_c1n_cs, self.q_c1n, self.fines_content, self.i_c, self.big_q, self.big_f = _calc_dependent_variables(self.sigma_v,
                                                                                                         self.sigma_veff,
                                                                                                         self.cpt.q_c,
                                                                                             self.cpt.f_s, self.p_a,
@@ -514,7 +514,7 @@ class BoulangerIdriss2014SoilProfile(object):  # TODO: validate this properly
         self.rd = calc_rd(self.depth, self.m_w)
         crr_unlimited = np.interp(self.depth, split_depths, self.sp.split['csr_n15'])
         self.crr_m7p5 = np.where(self.depth <= self.gwl, 4, crr_unlimited)
-        self.q_c1n_cs = calc_qc_1ncs_from_crr_m7p5(self.crr_m7p5)
+        self.q_c1n_cs = calc_q_c1n_cs_from_crr_m7p5(self.crr_m7p5)
         self.k_sigma = calc_k_sigma(self.sigma_veff, self.q_c1n_cs)
         self.msf = calc_msf(self.m_w, self.q_c1n_cs)
         self.crr = crr_m(self.k_sigma, self.msf, self.crr_m7p5)  # CRR at set magnitude
